import gtk, gobject, cairo, rsvg, math

( LEFT, RIGHT, CENTER, FULL ) = range( 4 )
( NORMAL, BOLD, ITALIC, UNDERLINE ) = ( 0, 1, 2, 4 )

# Define cairo equivelents here 

( CAIRO_HINT_STYLE_DEFAULT,
  CAIRO_HINT_STYLE_NONE,
  CAIRO_HINT_STYLE_SLIGHT,
  CAIRO_HINT_STYLE_MEDIUM,
  CAIRO_HINT_STYLE_FULL,
  CAIRO_HINT_STYLE_STROKE,
  CAIRO_HINT_NUM ) = range( 7 )
# STROKE uses a fine stroke to enhance further the weight of the text
# without bolding it, like a faux bold

( CAIRO_FONT_WEIGHT_NORMAL,
  CAIRO_FONT_WEIGHT_BOLD ) = range( 2 )

( CAIRO_FONT_SLANT_NORMAL, 
  CAIRO_FONT_SLANT_ITALIC ) = ( 0, 2 )


from time import sleep

# Create a GTK+ widget on which we will draw using Cairo
class Tile_1( gtk.DrawingArea ):

    # Draw in response to an expose-event
    __gsignals__ = { "expose-event": "override" }
    # Should probably add a signal event for motion notify and mouse buttons
    # Must be Tile Distinct
    
    # Handle the expose-event by drawing
    def do_expose_event( self, event ):

        # Create the cairo context
        cr = self.window.cairo_create()

        # Restrict Cairo to the exposed area; avoid extra work
        cr.rectangle( event.area.x, event.area.y, 
                event.area.width, event.area.height )
        cr.clip()

        icon = "dreamweaver.svg"
        title = "Adobe Dreamweaver 8"
        short_description = "Web design, development and\ncontent creation"
        rating = 3
        (width, height) = self.window.get_size()
        x = 0
        y = 0
        rect = ( x, y, width, height )
        my_Tile = Tile(cr, True,
                 title, 
                 icon,
                 short_description, 
                 rating, [], 
                 rect )

# I am heavily developing this at present, it is likely to get broken 
# between the hours of god knows when till forever and a day.
# the idea of this is to as re-usable as possible as a widget and pop it up
# on the pycairo/pygtk custom cell pages ;) 

# A Tile is a great user interface widget because it can display things,
# be interactive in its own right and is very descriptive visually and
# textually

# ASCII art
#-----------------------------------------------------------#
# +-----+ TITLE                              Button +-----+ #
# |  A  | Description                        Button +  B  + #
# +-----+ *****                              Status +-----+ #
#-----------------------------------------------------------#
# A Icon image
# B Animated image space
# some elements could be omitted, for instance buttons or icon
# or description can easily go. 

class Tile:
    def __init__( self, cr, render_now=False,  
                 title=None, 
                 icon=None, 
                 description=None, 
                 rating=None, 
                 buttons=[], 
                 rect=None ):
        self.cr = cr
        
        self.SetMaxButtons()
        self.buttons = buttons
        
        self.SetRect( rect )
        self.SetPadding()
        self.SetRating( rating )
        self.SetTitle( title )
        self.SetDescription( description )
        self.SetIcon( icon )
        
        if render_now:
            self.DrawTile()
        
    def SetTitle( self, title=None ):
        self.title = title
        self.title_text_ratio = 0.225 # 18px(title_font)/80px(height)
        self.title_max_size = 18
        
        self.title_size = self.title_text_ratio * self.height
        if self.title_size > self.title_max_size:
            self.title_size = self.title_max_size
            
        if self.rating:
            self.title_y = self.title_size * 1.1 # FIXME
        else:
            self.title_y = self.title_size 
        

    def SetIcon( self, icon=None ):
        self.icon = icon
        self.icon_height = self.height - ( 2 * self.padding )
        self.tile_info_x = self.height
        
    def SetDescription( self, description=None ):
        self.description = description
        self.description_height = self.height - (2*self.padding) - self.title_size - self.ratings_height
        self.description_ratio = 0.125
        self.description_max_size = 10
        self.description_size = self.description_ratio * self.height
        if self.description_size > self.description_max_size:
            self.description_size = self.description_max_size
        self.description_line_height = 1.2 * self.description_size
        self.description_rows = int(self.description_height / self.description_line_height)
        if self.rating:
            self.desc_y = self.title_y *1.56# FIXME    
        else:
            self.desc_y =  self.description_rows + self.description_size
        
    def SetRating( self, rating=None ):
        self.rating = rating
        
        if self.rating:
            self.ratings_ratio = 0.3
            self.ratings_height = self.ratings_ratio * self.height
        else:
            self.ratings_height = 0
        
    def SetRect( self, rect=None ):
        if not rect:
            rect = ( 0, 0, 500, 80 )
        ( self.x, self.y, self.width, self.height ) = rect
        
    def SetMaxButtons( self, buttons=4 ):
        self.max_buttons = buttons
        # Calculate text size for vertical buttons too
        
    def SetPadding( self, padding=5 ):
        self.padding = padding
        # Calculate padding dependent constants (realise the drawing area)
                    
    def AddButton( self, text, signal, properties, rect ):
        ( size, style, font, alignment, color ) = properties
        ( r, g, b ) = color
        if rect:
            ( x ,y, width, height ) = rect
        else:
            ( x ,y, width, height ) = ( None, None, None, None )
        self.buttons.append( x, y, signal, text, alignment, style, r, g, b )
        # Signal will probably be passed as data by the event handler
        
    # Thou shalst be merged with thy example code under hither
    def DrawTile( self ):
        # Move the canvas to the origin
        self.cr.translate( self.x, self.y )
        
        # Render each element separately
        self.DrawIcon()
        self.DrawTitle()
        self.DrawDescription()
        self.DrawRating()
        self.DrawButtons()
    
    def DrawIcon( self, x=None, y=None ):
        '''
        Simple image draw function, could equally be used for any kind of image
        image can be moved by x/y
        '''
        if not self.icon:
            return
        self.cr.save()
        if x and y:
            self.cr.translate( x, y )
        
        self.cr.translate( self.padding, self.padding )
        if self.icon.endswith( ".svg" ):
            # Render SVG files
            i_svg = rsvg.Handle( self.icon )
            self.icon_width = ( float( self.icon_height )/float( i_svg.props.height ) ) * float( i_svg.props.width )
            self.cr.translate( ( self.icon_height - self.icon_width )/2, 0 )
            self.cr.scale( float( self.icon_width )/float( i_svg.props.width ), 
                           float( self.icon_height )/float( i_svg.props.height ) )
            i_svg.render_cairo( self.cr )
        elif self.icon.endswith( ".png" ):
            # Render PNG files     
            image = cairo.ImageSurface.create_from_png ( self.icon )
            p_width, p_height = image.get_width(), image.get_height()
            self.icon_width = ( float( self.icon_height )/float( p_height ) ) * float( p_width )
            self.cr.translate( ( self.icon_height - self.icon_width )/2, 0 )
            self.cr.scale( self.icon_width/float( p_width ), 
                           self.icon_width/float( p_height ))
            self.cr.set_source_surface( image, 0, 0 );
            self.cr.paint()
        self.cr.restore()
        
    def DrawTitle( self ):
        self.Test()
        
    def DrawDescription( self ):
        pass
        
    def DrawRating( self ):
        if not self.rating:
            return
        if self.rating:
            r_svg = rsvg.Handle( "gold.svg" )
            
            tile_rating_y = self.icon_height - self.ratings_height + self.padding
            self.ratings_width = (r_svg.props.width/r_svg.props.height)*self.ratings_height
            
            self.cr.save()
            self.cr.translate( self.tile_info_x, tile_rating_y )
            self.cr.scale( float( self.ratings_width )/float( r_svg.props.width ), 
                           float( self.ratings_height )/float( r_svg.props.height ) )
            for x in range( 5 ):
                if x < self.rating:
                    r_svg.render_cairo( self.cr )
                else:
                    rating_radius = 3
                    #cr.translate((r_svg.props.width/2) - rating_radius, 0)
                    self.cr.arc( r_svg.props.width/2, 
                           r_svg.props.height/2, 
                           rating_radius, 0, 2.0 * math.pi )
                    self.cr.set_source_rgb( 0.73, 0.73, 0.71 )
                    self.cr.fill_preserve()
                    self.cr.set_source_rgb( 0.56, 0.56, 0.53 )
                    self.cr.set_line_width( 1 )
                    self.cr.stroke()
                self.cr.translate( r_svg.props.width , 0 )
            self.cr.restore()
            
    def DrawButtons( self ):
        if len(self.buttons) == 0:
            return
        
    def DrawText( self, text, properties, rect=None, effects=None):
        ( size, style, font, alignment, color ) = properties 
        if effects:
            ( shadow, stroke, hinting, opacity ) = effects
            ( shadow_offsetX, shadow_offsetY, shadow_opacity, shadow_size ) = shadow
            ( stroke_width, stroke_color, stroke_opacity ) = stroke
        self.cr.save()
        if rect:
            ( x ,y, width, height ) = rect
        else:
            ( x ,y, width, height ) = ( 0, 0, None, None )
        
        cr.set_font_size( size )
        if not font:
            font = "Bitstream Vera Sans"
        # styles 
        if style | BOLD:
            bold = CAIRO_FONT_WEIGHT_BOLD
        else:
            bold = CAIRO_FONT_WEIGHT_NORMAL
        if style | ITALIC:
            slant = CAIRO_FONT_SLANT_ITALIC
        else:
            slant = CAIRO_FONT_SLANT_NORMAL
            
        self.cr.set_font( font, slant, weight)
        (x_bearing, y_bearing, text_width, text_height, x_advance, y_advance) = cr.text_extents( text )
        (fascent, fdescent, fheight, fxadvance, fyadvance) = cr.font_extents()
                
        # line splitting, word wrapping and ellipsising
        lines = text.split("\n")
        
        # Justification and initial move/translation
        cursor_y = y - fyadvance
        if not hinting:
            hinting =  CAIRO_HINT_STYLE_STROKE

        if stroke_width and hinting == CAIRO_HINT_STYLE_STROKE:
            hinting = CAIRO_HINT_STYLE_FULL
            
        if style | UNDERLINE:
            uline_start = cursor_y*1.1
        
        for index,line in enumerate(lines):
            self.cr.save()
            self.cr.move_to(self.tile_info_x, cursor_y)
            self.cr.text_path( line )
            ( r, g, b ) = color
            self.cr.set_source_rgb( r, g, b )
            if hinting == CAIRO_HINT_STYLE_STROKE:
                self.cr.fill_preserve()
                ( r, g, b ) = color
                self.cr.set_source_rgb( r, g, b )
                self.cr.set_line_width( float(font_size/200) )
                self.cr.stroke()
            elif hinting < CAIRO_HINT_STYLE_STROKE:
                font_opts = self.cr.get_font_options()
                font_opts.set_hint_style(CAIRO_HINT_NUM - index)
                if stroke_width:
                    self.cr.fill_preserve()
                    ( r, g, b ) = stroke_color
                    self.cr.set_source_rgba( r, g, b, stroke_opacity )
                    self.cr.set_line_width( stroke_width )
                    self.cr.stroke()
                else:
                    self.cr.fill()
            
            cursor_y = cursor_y + fyadvance
            self.cr.restore()
            self.cr.save()
            if uline_start:
                (x_bearing, y_bearing, line_width, line_height, x_advance, y_advance) = cr.text_extents( line )
                uline_end = uline_start + line_width
                ( r, g, b ) = color
                self.cr.set_source_rgb( r, g, b )
                self.cr.move_to( uline_start, cursor_y*1.1 )
                self.cr.line_to( uline_end, cursor_y*1.1 )
                self.cr.set_line_width( font_size/20 )
                self.cr.stroke()
            self.cr.restore()
        self.cr.restore()            
        
def DrawText( cr, text, 
              xpos, ypos, 
              size, letter_spacing, word_spacing, line_spacing, 
              r, g, b, justify=LEFT, underline=False ):
        
        cr.set_font_size( size )
        #cr.set_source_rgb( r, g, b )
        #cr.fill_preserve()
        #cr.set_source_rgb( 0.56, 0.56, 0.53 )
        #cr.set_line_width( 50 )
        #cr.stroke()
        line_start = xpos
        
        fascent, fdescent, fheight, fxadvance, fyadvance = cr.font_extents()
        
        c_xpos = 0
        
        # Calculate position when right justification is used
        if justify == RIGHT:
            for cx, letter in enumerate( text ):
                xbearing, ybearing, width, height, xadvance, yadvance = \
                    cr.text_extents( letter )
                if letter == " ":
                    c_xpos = c_xpos + word_spacing
                else:
                    c_xpos = c_xpos + width + xbearing + letter_spacing
            uline_end = xpos
            xpos = xpos - c_xpos
            uline_start = xpos
        else:
            uline_end = None
            uline_start = xpos
        
        # Draw the text
        
        # On the advice of mirco muller, this is how I should do this ;)
        cr.save()
        
        # scan the text, if word wrap is enabled then use it
        # if elipsised text is enabled use it
        # otherwise split at every \n
        # itterate over the lines, adjusting position as required
        
        # else:
        lines = text.split("\n")
        
        for line in lines:
            cr.move_to(xpos,ypos)
            cr.text_path( line )
            cr.set_source_rgb( r, g, b )
            #cr.fill()
            cr.fill_preserve()
            cr.set_source_rgb( r, g, b )
            cr.set_line_width( 0.2 )
            cr.stroke()
            ypos = ypos + line_spacing
        
        # Urgh!
        #for cx, letter in enumerate( text ):
        #    if letter == " ":
        #        xpos = xpos + word_spacing
        #        continue
        #    if letter == "\n":
        #        xpos = line_start
        #        ypos = ypos + line_spacing
        #        continue
        #    xbearing, ybearing, width, height, xadvance, yadvance = \
        #            cr.text_extents( letter )
        #    cr.move_to( xpos, ypos )
        #    #cr.show_text( letter )
        #    cr.text_path( letter )
        #    xpos = xpos + width + xbearing + letter_spacing
        #    if not uline_end:
        #        uline_end = xpos
        
        
        cr.restore()
        
        # Draw the underline
        cr.save()
        # unline_end is wrong!!
        if underline:
            cr.move_to( uline_start, ypos*1.1 )
            cr.line_to( uline_end, ypos*1.1 )
            cr.set_line_width( 1 )
            cr.stroke()
        cr.restore()      
        
def DrawTile( cr, icon, title, short_description, rating=None, max_buttons=4, x=0, y=0, width=500, height=80 ):
    cr.translate(x,y)
    # Fill the background with white
    
    cr.save()
    cr.set_source_rgb(1, 1, 1)
    cr.rectangle(0, 0, width, height)
    cr.fill()
    cr.restore()
    
    # Padding & rating
    padding = 5
    
    # Draw icon
    if icon.endswith(".svg"):
        # Render SVG files
        i_svg = rsvg.Handle(icon)
        i_width = height - 2 * padding
        i_height = (float(i_width)/float(i_svg.props.width)) * float(i_svg.props.height)
        
        cr.save()
        cr.translate(padding,padding)
        cr.scale(float(i_width)/float(i_svg.props.width),
                 float(i_height)/float(i_svg.props.height))
        i_svg.render_cairo(cr)
        cr.restore()
    elif icon.endswith(".png"):
        # Render PNG files
        cr.save()           
        cr.translate(padding,padding)
        image = cairo.ImageSurface.create_from_png (icon)
        p_width, p_height = image.get_width(), image.get_height()
        i_width = height - 2 * padding
        i_height = (float(i_width)/float(p_width)) * float(p_height)
        cr.scale(float(i_width)/float(p_width),
                 float(i_height)/float(p_height))
        cr.set_source_surface(image, 0, 0);
        cr.paint()
        cr.restore()
        
    #Set the start point for the other elements
    tile_info_x = i_width + 2*padding
    
    if rating:
        # Draw rating *'s
        r_width = i_width/3
        r_height = i_height/3
        r_svg = rsvg.Handle( "gold.svg" )
        
        tile_rating_y = i_height - r_height + padding
        
        cr.save()
        cr.translate( tile_info_x, tile_rating_y )
        cr.scale( float( r_width )/float( r_svg.props.width ), 
                 float( r_height )/float( r_svg.props.height ) )
        for x in range( 5 ):
            if x < rating:
                r_svg.render_cairo( cr )
             #   cr.translate(r_svg.props.width, 0)
            else:
                rating_radius = 3
                #cr.translate((r_svg.props.width/2) - rating_radius, 0)
                cr.arc( r_svg.props.width/2, 
                       r_svg.props.height/2, 
                       rating_radius, 0, 2.0 * math.pi )
                cr.set_source_rgb( 0.73, 0.73, 0.71 )
                cr.fill_preserve()
                cr.set_source_rgb( 0.56, 0.56, 0.53 )
                cr.set_line_width( 1 )
                cr.stroke()
                
            cr.translate( r_svg.props.width , 0 )
        cr.restore()
        
        # Calculate some differences
        title_font_size = height / 3.5
        tile_title_y = title_font_size* 1.1
        tile_desc_y = tile_title_y*1.56
        desc_font_size = ( i_height - title_font_size - r_height ) / 2.2
    else:
        title_font_size = i_height / 2
        tile_title_y = title_font_size
        desc_font_size = ( i_height*0.9 ) / 4
        tile_desc_y = ( height / 2 ) + desc_font_size
        
    # Draw title
    line_spacing = title_font_size + 1
    word_spacing = title_font_size / 2
    
    cr.save()
    DrawText( cr, title, 
                  tile_info_x, tile_title_y, 
                  title_font_size, 1.05, word_spacing, line_spacing, 
                  0, 0, 0 )
    cr.restore()
    
    # Draw description
    line_spacing = desc_font_size + 1
    word_spacing = desc_font_size / 2
    
    cr.save()
    DrawText( cr, short_description, 
             tile_info_x, tile_desc_y, 
             desc_font_size, 1.02, word_spacing, line_spacing, 
             0.73, 0.73, 0.71 )
    cr.restore()
    
    buttons = ( "Install", "Buy" )
    right_edge = width - padding - ( 2*height )/3
    if not max_buttons:
        max_buttons = len( buttons )+1
    button_font_size = i_height / ( max_buttons + 1 )
    # Draw a button
    line_spacing = button_font_size + 3
    word_spacing = button_font_size / 2
    
    status_font_size = i_height / ( max_buttons * 1.2 )
    status_text = "Downloading 45%"
    
    cr.save()
    if status_text:
        status = 1

    for bx, button in enumerate( buttons ):
        if bx < max_buttons - status:
            DrawText( cr, button, 
                          right_edge, ( ( bx+1 )*line_spacing ), 
                          button_font_size, 1.03, word_spacing, line_spacing, 
                          0.12, 0.29, 0.53, RIGHT, True )
    
    line_spacing = status_font_size + 3
    word_spacing = status_font_size / 2
    
    DrawText( cr, status_text, 
                  right_edge, i_height+( padding/3 ), 
                  status_font_size, 1.06, word_spacing, line_spacing, 
                  0.73, 0.73, 0.71, RIGHT )
    
    cr.restore()
    

            
# GTK mumbo-jumbo to show the widget in a window and quit when it's closed
def run( Widget ):
    window = gtk.Window()
    window.connect( "delete-event", gtk.main_quit )
    widget = Widget()
    window.resize( 580, 200 )
    widget.show()
    window.add( widget )
    window.present()
    gtk.main()

if __name__ == "__main__":
    run( Tile_1 )